@@ -71,6 +71,8 @@ class ExpressionEdit : public QPlainTextEdit {
 		QString current_history;
 		int history_index, undo_index;
 		int tabbed_index;
+		int previous_pos;
+		QString previous_text;
 		bool enable_tab;
 		
 		CompletionData *cdata;
@@ -136,7 +138,7 @@ class ExpressionEdit : public QPlainTextEdit {
 		void updateCompletion();
 		void setExpression(std::string);
 		void setExpression(const QString &str);
-		void blockCompletion(bool = true);
+		void blockCompletion(bool = true, bool = true);
 		void blockParseStatus(bool = true);
 		void blockUndo(bool = true);
 		void hideCompletion();
