@@ -388,49 +388,6 @@ void HistoryView::addResult(std::vector<std::string> values, std::string express
 	QString str;
 	if(!expression.empty() || !parse.empty()) {
 		str += QString("<tr><td colspan=\"2\" style=\"padding-bottom: %1 px; padding-top: 0px; border-top: 0px none %2; text-align:left\">").arg(paste_h / 4).arg(textColor().name());
-		if(!expression.empty() && (settings->history_expression_type > 0 || parse.empty())) {
-			if(!parse.empty() && settings->history_expression_type > 1 && parse != expression) {
-				str += QString("<a name=\"e%1\" style=\"text-decoration: none\">").arg(initial_load ? (int) index : settings->v_expression.size());
-				str += QString::fromStdString(expression).toHtmlEscaped();
-				str += "</a>";
-				if(settings->color == 2) str += "&nbsp; <i style=\"color: #AAAAAA\">";
-				else str += "&nbsp; <i style=\"color: #666666\">";
-				str += QString("<a name=\"a%1\" style=\"text-decoration: none\">").arg(initial_load ? (int) index : settings->v_expression.size());
-				if(pexact) str += "= ";
-				else str += SIGN_ALMOST_EQUAL " ";
-				str += QString::fromStdString(replace_parse_colors(parse));
-				str += "</a>";
-				str += "</i>";
-			} else {
-				str += QString("<a name=\"%1\" style=\"text-decoration: none\">").arg(initial_load ? (int) index : settings->v_expression.size());
-				str += QString::fromStdString(expression).toHtmlEscaped();
-				str += "</a>";
-			}
-		} else {
-			str += QString("<a name=\"%1\" style=\"text-decoration: none\">").arg(initial_load ? (int) index : settings->v_expression.size());
-			if(!pexact) str += SIGN_ALMOST_EQUAL " ";
-			str += QString::fromStdString(parse);
-			str += "</a>";
-			if(b_parse_error && !expression.empty()) {
-				str += "&nbsp;&nbsp;&nbsp; ";
-				if(settings->color == 2) str += "<i style=\"color: #AAAAAA\">";
-				else str += "<i style=\"color: #666666\">";
-				str += QString("<a name=\"e%1\" style=\"text-decoration: none\">").arg(initial_load ? (int) index : settings->v_expression.size());
-				str += QString::fromStdString(replace_parse_colors(expression));
-				str += "</a>";
-				str += "</i>";
-			}
-		}
-		if(initial_load && settings->v_protected[index]) {
-			if(has_lock_symbol < 0) {
-				QFontDatabase database;
-				if(database.families(QFontDatabase::Symbol).contains("Noto Color Emoji")) has_lock_symbol = 1;
-				else has_lock_symbol = 0;
-			}
-			if(has_lock_symbol) str += " <small><sup>🔒</sup></small>";
-			else str += " <small><sup>[P]</sup></small>";
-		}
-		str += "</td></tr>";
 		if(!initial_load) {
 			settings->v_expression.push_back(expression);
 			settings->v_parse.push_back(parse);
@@ -447,6 +404,50 @@ void HistoryView::addResult(std::vector<std::string> values, std::string express
 				settings->v_value[settings->v_value.size() - 1].push_back(dual_approx && i == 0 ? settings->history_answer.size() - 1 : settings->history_answer.size());
 			}
 		}
+		gsub("</i>", "<img src=\"data://img1px.png\" width=\"1\"/></i>", parse);
+		if(!expression.empty() && (settings->history_expression_type > 0 || parse.empty())) {
+			if(!parse.empty() && settings->history_expression_type > 1 && parse != expression) {
+				str += QString("<a name=\"e%1\" style=\"text-decoration: none\">").arg(initial_load ? (int) index : settings->v_expression.size() - 1);
+				str += QString::fromStdString(expression).toHtmlEscaped();
+				str += "</a>";
+				if(settings->color == 2) str += "&nbsp; <i style=\"color: #AAAAAA\">";
+				else str += "&nbsp; <i style=\"color: #666666\">";
+				str += QString("<a name=\"a%1\" style=\"text-decoration: none\">").arg(initial_load ? (int) index : settings->v_expression.size() - 1);
+				if(pexact) str += "= ";
+				else str += SIGN_ALMOST_EQUAL " ";
+				str += QString::fromStdString(replace_parse_colors(parse));
+				str += "</a>";
+				str += "</i>";
+			} else {
+				str += QString("<a name=\"%1\" style=\"text-decoration: none\">").arg(initial_load ? (int) index : settings->v_expression.size() - 1);
+				str += QString::fromStdString(expression).toHtmlEscaped();
+				str += "</a>";
+			}
+		} else {
+			str += QString("<a name=\"%1\" style=\"text-decoration: none\">").arg(initial_load ? (int) index : settings->v_expression.size() - 1);
+			if(!pexact) str += SIGN_ALMOST_EQUAL " ";
+			str += QString::fromStdString(parse);
+			str += "</a>";
+			if(b_parse_error && !expression.empty()) {
+				str += "&nbsp;&nbsp;&nbsp; ";
+				if(settings->color == 2) str += "<i style=\"color: #AAAAAA\">";
+				else str += "<i style=\"color: #666666\">";
+				str += QString("<a name=\"e%1\" style=\"text-decoration: none\">").arg(initial_load ? (int) index : settings->v_expression.size() - 1);
+				str += QString::fromStdString(replace_parse_colors(expression));
+				str += "</a>";
+				str += "</i>";
+			}
+		}
+		if(initial_load && settings->v_protected[index]) {
+			if(has_lock_symbol < 0) {
+				QFontDatabase database;
+				if(database.families(QFontDatabase::Symbol).contains("Noto Color Emoji")) has_lock_symbol = 1;
+				else has_lock_symbol = 0;
+			}
+			if(has_lock_symbol) str += " <small><sup>🔒</sup></small>";
+			else str += " <small><sup>[P]</sup></small>";
+		}
+		str += "</td></tr>";
 	} else if(!initial_load && !settings->v_result.empty()) {
 		for(size_t i = values.size(); i > 0; i--) {
 			settings->v_result[settings->v_result.size() - 1].insert(settings->v_result[settings->v_result.size() - 1].begin(), values[i - 1]);
@@ -507,7 +508,28 @@ void HistoryView::addResult(std::vector<std::string> values, std::string express
 		else if(b_exact == 0) str += SIGN_ALMOST_EQUAL " ";
 		str += QString::fromStdString(values[i]);
 		if(!image.isEmpty() && i == values.size() - 1 && w * 2 <= width()) {
-			str += QString("<img src=\"data://img1px.png\" width=\"2\"/><img valign=\"top\" src=\"%1\" height=\"%2\"/>").arg(image).arg(fm.ascent());
+			str += QString("<img src=\"data://img1px.png\" width=\"2\"/><img valign=\"top\" src=\"%1\" height=\"%2\"").arg(image).arg(fm.ascent());
+			CALCULATOR->setExchangeRatesUsed(-100);
+			int i = CALCULATOR->exchangeRatesUsed();
+			CALCULATOR->setExchangeRatesUsed(-100);
+			if(i > 0) {
+				QString sources_str;
+				int n = 0;
+				if(i & 0b0001) {sources_str += "\\n"; sources_str += QString::fromStdString(CALCULATOR->getExchangeRatesUrl(1)); n++;}
+				if(i & 0b0010) {sources_str += "\\n"; sources_str += QString::fromStdString(CALCULATOR->getExchangeRatesUrl(2)); n++;}
+				if(i & 0b0100) {sources_str += "\\n"; sources_str += QString::fromStdString(CALCULATOR->getExchangeRatesUrl(3)); n++;}
+				if(i & 0b1000) {sources_str += "\\n"; sources_str += QString::fromStdString(CALCULATOR->getExchangeRatesUrl(4)); n++;}
+				if(n > 0) {
+					str += " alt=\"";
+					str += tr("Exchange rate source(s):", "", n);
+					str += sources_str;
+					str += "\\n(";
+					str += tr("updated %1", "", n).arg(QString::fromStdString(QalculateDateTime(CALCULATOR->getExchangeRatesTime(CALCULATOR->exchangeRatesUsed())).toISOString()));
+					str += ")";
+					str += "\"";
+				}
+			}
+			str += "/>";
 		}
 		str += "</a></td></tr>";
 		i_answer_pre = i_answer;
@@ -579,6 +601,33 @@ void HistoryView::addMessages() {
 }
 void HistoryView::mouseMoveEvent(QMouseEvent *e) {
 	QString str = anchorAt(e->pos());
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 8, 0))
+	bool b_tooltip = false;
+	if(str.isEmpty() && e->pos().x() > width() - 150) {
+		if(document()->documentLayout()->imageAt(e->pos()).indexOf(":/data/flags") == 0) {
+			QPoint p(viewport()->mapFromParent(e->pos()));
+			for(int n = 0; n < 5; n++) {
+				QTextCursor cur = cursorForPosition(p);
+				cur.movePosition(QTextCursor::StartOfBlock, QTextCursor::KeepAnchor);
+				QString str = cur.selection().toHtml();
+				int i = str.lastIndexOf(" alt=\"");
+				if(i >= 0) {
+					i += 6;
+					int i2 = str.indexOf("\"", i + 1);
+					if(i2 > i) {
+						str = str.mid(i, i2 - i);
+						str.replace("\\n", "\n");
+						QToolTip::showText(mapToGlobal(e->pos()), str);
+						b_tooltip = true;
+					}
+					break;
+				}
+				p.rx() += QFontMetrics(font()).ascent() / 2;
+			}
+		}
+	}
+	if(!b_tooltip) QToolTip::hideText();
+#endif
 	if(str.isEmpty()) viewport()->setCursor(Qt::IBeamCursor);
 	else viewport()->setCursor(Qt::PointingHandCursor);
 	QTextEdit::mouseMoveEvent(e);
@@ -687,7 +736,9 @@ void HistoryView::inputMethodEvent(QInputMethodEvent *e) {
 void HistoryView::editClear() {
 	for(size_t i1 = 0; i1 < settings->v_protected.size(); i1++) {
 		if(!settings->v_protected[i1]) {
-			if(i1 == 0) settings->current_result = NULL;
+			if(i1 == settings->v_protected.size() - 1) {
+				settings->current_result = NULL;
+			}
 			settings->v_delexpression[i1] = true;
 			int index = s_text.indexOf("<a name=\"" + QString::number(i1) + "\"");
 			if(index < 0) index = s_text.indexOf("<a name=\"e" + QString::number(i1) + "\"");
@@ -843,7 +894,7 @@ void HistoryView::indexAtPos(const QPoint &pos, int *expression_index, int *resu
 	if(value_index) *value_index = -1;
 	QString sref = anchorAt(pos);
 	if(sref.isEmpty()) {
-		QTextCursor cur = cursorForPosition(pos);
+		QTextCursor cur = cursorForPosition(viewport()->mapFromParent(pos));
 		cur.movePosition(QTextCursor::StartOfBlock, QTextCursor::KeepAnchor);
 		QString str = cur.selection().toHtml();
 		int i = str.lastIndexOf("<a name=\"");
@@ -854,7 +905,7 @@ void HistoryView::indexAtPos(const QPoint &pos, int *expression_index, int *resu
 		}
 	}
 	if(sref.isEmpty()) {
-		QTextCursor cur = cursorForPosition(pos);
+		QTextCursor cur = cursorForPosition(viewport()->mapFromParent(pos));
 		cur.movePosition(QTextCursor::EndOfBlock, QTextCursor::MoveAnchor);
 		cur.movePosition(QTextCursor::StartOfBlock, QTextCursor::KeepAnchor);
 		QString str = cur.selection().toHtml();
@@ -971,7 +1022,7 @@ void HistoryView::editFind() {
 	}
 	searchDialog = new QDialog(this);
 	QVBoxLayout *box = new QVBoxLayout(searchDialog);
-	if(settings->always_on_top) searchDialog->setWindowFlags(windowFlags() | Qt::WindowStaysOnTopHint);
+	if(settings->always_on_top) searchDialog->setWindowFlags(searchDialog->windowFlags() | Qt::WindowStaysOnTopHint);
 	searchDialog->setWindowTitle(tr("Search"));
 	QGridLayout *grid = new QGridLayout();
 	grid->addWidget(new QLabel(tr("Text:"), this), 0, 0);
